﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace _485USBSerial
{
    public partial class Form1 : Form
    {
        string[] ports;        

        SerialPortInterface currentSerialDevice;
        private Thread writeThread = null;

        public delegate void ThreadSafeAddConsoleTextCallback(string text);
        public delegate void threadSafeToggleSendButtonCallback(Boolean toggle);



        public Form1()
        {
            InitializeComponent();
            currentSerialDevice = new SerialPortInterface(this);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            refreshPorts();
            baudDropdown.SelectedIndex = baudDropdown.Items.Count-1;
            stopBitsDropdown.SelectedIndex = 0;
            parityDropdown.SelectedIndex = 0;
            dataBitsDropdown.SelectedIndex = dataBitsDropdown.Items.Count - 1;
            flowControlDropdown.SelectedIndex = 0;
            versionLabel.Text = String.Format("Version: {0}", typeof(Form1).Assembly.GetName().Version);
        }

        

        public void refreshPorts()
        {
            commDevicesDropdown.Items.Clear(); // clear the comboBox

            ports = SerialPort.GetPortNames();

            commDevicesDropdown.Items.AddRange(ports);
        }

        public void updateSerialConsole(string data)
        {
            consoleTextBox.Text += data;
        }

        private void readPortCallback()
        {
            currentSerialDevice.outputToSerialConsole(this);
        }

        public void sendText()
        {
            if (textEntry.Text.Length > 0)
            {
                bool writeResult = currentSerialDevice.WriteString(textEntry.Text + Environment.NewLine);

                if (writeResult)
                {
                    updateSerialConsole(textEntry.Text + "\n");
                    textEntry.Text = "";
                }

            }
        }



        private void refreshButton_Click(object sender, EventArgs e)
        {
            refreshPorts();
        }

        private void textEntry_KeyPress(object sender, KeyPressEventArgs e)
        {
            
            if (e.KeyChar == (char)13) //the enter key was pressed
                sendText();
            
        }

        private void commDevicesDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            

            if (commDevicesDropdown.Text != "")
            {
                currentSerialDevice.PortName = commDevicesDropdown.Text;
            }
            
        }

        private void connectSerialDevice()
        {
            if (currentSerialDevice.Open()) //if the port was opened successfully
            {
                currentConnectionLabel.Text = currentSerialDevice.PortName;
            }

        }

       
        private void sendFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog fd = new OpenFileDialog();
            if (fd.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                byte[] fileBytes = File.ReadAllBytes(fd.FileName);

                writeThread = new Thread(() => sendBytes(fileBytes));
                threadSafeToggleSendButton(false);
                writeThread.Start();
            }
        }

        private void sendBytes(byte[] byteArray)
        {
            currentSerialDevice.WriteBytes(byteArray);
            threadSafeToggleSendButton(true);
        }

        private void asciiRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            string[] newLines;
            if (asciiRadioButton.Checked)
            {
                newLines = new string[consoleTextBox.Lines.Length];
                for (int i = 0; i < consoleTextBox.Lines.Length; i++)
                    newLines[i] = toAscii(consoleTextBox.Lines[i]);
                consoleTextBox.Lines = newLines;
            }
            else
            {
                newLines = new string[consoleTextBox.Lines.Length];
                for (int i = 0; i < consoleTextBox.Lines.Length; i++)
                    newLines[i] = toHex(consoleTextBox.Lines[i]);
                consoleTextBox.Lines = newLines;
            }
        }

        public string toHex(string ascii)
        {
            char[] chars = ascii.ToCharArray();
            string newStr = "";
            for (int i = 0; i < chars.Length; i++)
            {
                string h = String.Format("{0:X}", (int)chars[i]) + " ";
                if (h.Length < 3)
                    h = "0" + h;
                newStr += h;
            }
            return newStr;
        }

        public string toAscii(string hex/*space separated*/)
        {
            string newStr = "";
            for (int i = 0; i < hex.Length; i += 3)
                newStr += Convert.ToChar(Convert.ToUInt32(hex.Substring(i, 2), 16));
            return newStr;
        }

        private void sendTextButton_Click(object sender, EventArgs e)
        {
            threadSafeToggleSendButton(false);
            sendText();
            threadSafeToggleSendButton(true);
        }

        // TODO: kick off read thread here to reduce thread overhead
        private void textEntry_TextChanged(object sender, EventArgs e)
        {

        }

        private void textEntry_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.V && e.Modifiers == Keys.Control)
            {
                textEntry.Text = Clipboard.GetText();
            }
        }

        private void dataBitsDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (dataBitsDropdown.Text != "")
            {
                currentSerialDevice.DataBits = int.Parse(dataBitsDropdown.Text);
            }
        }

        private void parityDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (parityDropdown.Text != "")
            {
                switch (parityDropdown.Text)
                {
                    case "None":
                        currentSerialDevice.Parity = Parity.None;
                        break;
                    case "Odd":
                        currentSerialDevice.Parity = Parity.Odd;
                        break;
                    case "Even":
                        currentSerialDevice.Parity = Parity.Even;
                        break;
                    default:
                        break;
                }
            }
        }

        private void flowControlDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (flowControlDropdown.Text != "")
            {
                switch (flowControlDropdown.Text)
                {
                    default:

                        break;
                }
            }
        }

        private void stopBitsDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (stopBitsDropdown.Text != "")
            {
                switch (stopBitsDropdown.Text)
                {
                    case "1":
                        currentSerialDevice.StopBits = StopBits.One;
                        break;
                    case "2":
                        currentSerialDevice.StopBits = StopBits.Two;
                        break;
                    default:
                        break;
                }
            }
        }

        private void baudDropdown_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (baudDropdown.Text != "")
            {
                currentSerialDevice.BaudRate = int.Parse(baudDropdown.Text);
            }
        }

        private void hexRadioButton_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            connectSerialDevice();
        }

        public void threadSafeAddConsoleText(string text)
        {
            if (this.consoleTextBox.InvokeRequired)
            {
                ThreadSafeAddConsoleTextCallback d = new ThreadSafeAddConsoleTextCallback(threadSafeAddConsoleText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                if (hexRadioButton.Checked)
                    consoleTextBox.Text += toHex(text);
                else
                    consoleTextBox.Text += text;
            }
        }

        public void threadSafeToggleSendButton(Boolean toggle)
        {
            if (this.consoleTextBox.InvokeRequired)
            {
                threadSafeToggleSendButtonCallback d = new threadSafeToggleSendButtonCallback(threadSafeToggleSendButton);
                this.Invoke(d, new object[] { toggle });
            }
            else
            {
                if (toggle)
                    sendTextButton.Enabled = true;
                else
                    sendTextButton.Enabled = false;
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            bool isOpen = currentSerialDevice.Close();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            consoleTextBox.Clear();
        }
    }
}
