﻿using System;
using System.IO.Ports;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Threading;

namespace _485USBSerial
{
    class SerialPortInterface
    {
        private SerialPort m_serialPort;// = new SerialPort();
        private int m_baudRate;// = 9600;
        private int m_dataBits;// = 8;
        private Handshake m_handshake = Handshake.None;
        private Parity m_parity;// = Parity.None;
        private string m_portName;// = "COM1";
        private StopBits m_stopBits; // = StopBits.One;

        private string tString = string.Empty;
        private string hexString = string.Empty;
        private string asciiString = string.Empty;

        private byte m_terminator = 0x4;

        private Thread readThread = null;
        private Form1 form;

        private bool keepReading = true;

        public SerialPortInterface(Form1 form)
        {
            m_handshake = Handshake.None;
            m_serialPort = new SerialPort();
            this.form = form;
        }

        public SerialPortInterface(string portName, int baudRate, int dataBits, Parity parity, StopBits stopBits, Form1 form)
        {
            m_portName = portName;
            m_baudRate = baudRate;
            m_dataBits = dataBits;
            m_parity = parity;
            m_stopBits = stopBits;
            m_handshake = Handshake.None;
            m_serialPort = new SerialPort();
            this.form = form;
        }

        public SerialPortInterface(string portName, Form1 form)
        {
            m_portName = portName;
            m_serialPort = new SerialPort();
            this.form = form;
        }

        public bool Open()
        {
            try
            {
                if (m_serialPort.IsOpen)
                {
                    keepReading = false;
                    Close();
                    readThread.Join();
                }
                m_serialPort.BaudRate = m_baudRate;
                m_serialPort.DataBits = m_dataBits;
                m_serialPort.Handshake = m_handshake;
                m_serialPort.Parity = m_parity;
                m_serialPort.PortName = m_portName;
                m_serialPort.StopBits = m_stopBits;
                m_serialPort.Open();



                readThread = new Thread(readThreadEntry);
                readThread.Start();
            }
            catch { return false; }
            return true;
        }

        public bool Close()
        {
            try
            {
                m_serialPort.Close();
            }
            catch
            {
                return false;
            }
            return true;
        }

        public bool WriteString(string data)
        {
            try
            {
                m_serialPort.Write(data);
            }
            catch(InvalidOperationException ioe)
            {
                Console.WriteLine("A Invalid Operation exception occurred:",ioe.ToString(),".\n");
                return false;
            }
            catch(ArgumentNullException ane)
            {
                Console.WriteLine("A Argument Null exception occurred:",ane.ToString(),".\n");
                return false;
            }
            catch(TimeoutException toe)
            {
                Console.WriteLine("A timeout exception occurred",toe.ToString(),".\n");
                return false;
            }

            return true;
        }

        
        public bool WriteBytes(byte[] byteArray)
        {
            int chunkSize=65535;
            int byteCount=0;
            int offset=0;

            if(byteArray.Length < chunkSize)
            {
                chunkSize = byteArray.Length;
            }
            try
            {
                while (byteCount < byteArray.Length)
                {
                    m_serialPort.Write(byteArray, offset, chunkSize);
                    offset += chunkSize;
                    byteCount += chunkSize;
                    if( (byteCount + chunkSize) > byteArray.Length)
                    {
                        chunkSize = byteArray.Length - byteCount;
                    }
                }           
            }
            catch (InvalidOperationException ioe)
            {
                Console.WriteLine("A Invalid Operation exception occurred",ioe.ToString(),".\n");
                return false;
            }
            catch (ArgumentNullException ane)
            {
                Console.WriteLine("A Argument Null exception occurred",ane.ToString(),".\n");
                return false;
            }
            catch (TimeoutException toe)
            {
                Console.WriteLine("A timeout exception occurred",toe.ToString(),".\n");
                return false;
            }

            return true;
        }

        public void readThreadEntry()
        {
            // initialize buffer to hold received data
            byte[] buffer = new byte[m_serialPort.ReadBufferSize];
            while (keepReading)
            {
                int bytesRead = 0;
                try
                {
                    bytesRead = m_serialPort.Read(buffer, 0, buffer.Length);

                    tString = Encoding.ASCII.GetString(buffer, 0, bytesRead);
                    asciiString = tString;
                    asciiToHex(ref hexString, asciiString);

                    // UNUSED in this sample application, but included for the purpose of example
                    //check if string contains terminating character 
                    if (tString.IndexOf((char)m_terminator) > -1)
                    {
                        //if tString does contain terminator, we cannot assume that it is the last char received
                        string workingString = tString.Substring(0, tString.IndexOf((char)m_terminator));

                        //remove the data up to the terminator from tString
                        tString = tString.Substring(tString.IndexOf((char)m_terminator));

                        Console.WriteLine(workingString);
                    }
                    if (bytesRead > 0)
                        form.threadSafeAddConsoleText(tString);
                }
                catch (Exception ex)
                {
                    if (bytesRead < 1)
                        keepReading = false;
                }
            }
            keepReading = true;
        }        

        public void outputToSerialConsole(Form1 sender)
        {
            while (true)
            {
                if (tString != "")
                {
                    sender.updateSerialConsole(tString);
                    tString = "";
                    break;
                }
                else
                {
                    System.Threading.Thread.Sleep(500);
                }

            }
        }

        private void asciiToHex(ref string hexStringOut, string asciiStringIn)
        {
            foreach (char asciiChar in asciiStringIn)
            {
                int charValue = Convert.ToInt32(asciiChar);
                hexStringOut += String.Format("{0:X}", charValue);
            }
        }

        public int BaudRate { get { return m_baudRate; } set { m_baudRate = value; }}
        public int DataBits { get { return m_dataBits; } set { m_dataBits = value; }}
        public Handshake Handshake { get { return m_handshake; } set { m_handshake = value; }}
        public Parity Parity { get { return m_parity; } set { m_parity = value; }}
        public string PortName { get { return m_portName; } set { m_portName = value; }}
        public StopBits StopBits { get { return m_stopBits; } set { m_stopBits = value; }}
        public string TString { get { return tString; } set { tString = value; }}
        public string AsciiString { get { return asciiString; } set { asciiString = value; }}
        public string HexString { get { return hexString; } set { hexString = value; }}
    }
}
